import 'dart:async';
import 'dart:typed_data';
import 'world.dart';

class ModeratorSites {
  //init site from startup list
  //or join site
  interzoneDisk? disk;
  Map<String, world> _sites = {};
  bool get _dirty {
    for (var s in _sites.values) {
      if (s.isDirty) return true;
    }
    return false;
  }

  void unDirty() {
    for (var s in _sites.values) {
      s.unDirty();
    }
  }

  void init(interzoneDisk _io) {
    disk = _io;
    //read site specific things
  }

  List<String> siteKeys = [];
  String homeSite = '';
  String lastPushedCID = '';
  int lastPushedCIDByteLength = 0;

  List<world> get allSites => _sites.values.toList();
  List<ModeratorEntry> threadByShortLink(String shortLink) {
    for (var i in allSites) {
      final l = i.threadByShortLink(shortLink);
      if (l.isNotEmpty) return l;
    }
    return [];
  }

  Future<bool> followSite(String siteName, String url, String nick) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (_sites[siteName] != null) return true;

    if (nick.isEmpty) nick = _sites[homeSite]!.currentNick;

    await disk!.init(siteName + 'Data', false);
    final world iz = world();
    iz.appSplashImage = siteName;
    return iz.init(siteName, url, nick, disk!).then((value) {
      if (value == null) return false;
      _sites[siteName] = iz;
      siteKeys.add(siteName);
      if (homeSite.isEmpty) homeSite = siteName;
      ModeratorEntrySet _es = ModeratorEntrySet();

      _sites.forEach((key, value) {
        value.messages.all.forEach((element) {
          element.siteUrl = key;
          _es.add(element);
        });
      });
      _combinedEntrySetCache = _es;

      return true;
    });
  }

  bool switchHomeSite(String siteName, String url) {
    //gets url
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (homeSite == siteName) return true;
    if (!siteKeys.contains(siteName)) return false;
    homeSite = siteName;
    //TODO flush previous homesite
    _sites[siteName]!.ping();
    return true;
  }

  bool ping() {
    allSites.forEach((element) {
      element.ping();
    });
    return true;
  }

  world? findSite(String site) => _sites[site];
  world? get findHomeSite => findSite(homeSite);
  String get nick {
    if (findHomeSite != null) return findHomeSite!.currentNick;
    return 'deadHomeSitebeef';
  }

  ModeratorEntrySet _combinedEntrySetCache = ModeratorEntrySet();

  ModeratorEntrySet get combinedEntrySet {
    if (!_dirty && _combinedEntrySetCache.all.isNotEmpty) {
      return _combinedEntrySetCache;
    }
    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;
    unDirty();
    return _combinedEntrySetCache;
  }

  ModeratorEntrySet? siteEntrySet(String site) {
    ModeratorEntrySet _es = ModeratorEntrySet();
    if (_sites[site] == null) return _es;
    return _sites[site]?.messages;
  }

  Future<bool> setNick(String nick, String site) async {
    if (site.isEmpty) site = homeSite;
    return await _sites[site]!.setUserNickAndImage(nick, Uint8List(0));
  }

  Future<bool> setIpfsCredentials(
      String url, String k1, String k2, String site) async {
    if (site.isEmpty) {
      _sites.values.forEach((element) {
        element.setIpfsCredentials(k1, k2, url);
      });
      return true;
    }
    return await _sites[site]!.setIpfsCredentials(k1, k2, '');
  }

  Future<bool> unfollowSite(String siteName, String url, String nick) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    //todo flush caches etc
    _sites[siteName]!.terminate();
    _sites.remove(siteName);
    siteKeys.remove(siteName);
    if (homeSite == siteName) {
      homeSite = siteKeys.first;
    }

    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;

    return true;
  }

  /*Future<bool> deleteEntry(String shortLink) async {
    if (_sites[homeSite] == null) return false;
    _sites[homeSite]!.deleteEntry(shortLink);
    return true;
  }*/

  Future<bool> dropEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = _sites[surl]!.dropEntry(shortLink);
    if (li == null) return false;
    return true;
  }

  Future<bool> muteAuthor(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    return _sites[surl]!.muteAuthor(target);
  }

  Future<bool> shareEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = await _sites[surl]!.shareEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<bool> likeEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final li = _sites[target.siteUrl]!.likeEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<bool> postEntry(
      String shortLink, String text, Uint8List attachment) async {
    if (_sites[homeSite] == null) return false;

    final pe = await _sites[homeSite]!
        .newThread(homeSite, text, attachment, '', '', '');
    if (pe != null && pe.isEmpty) return false;
    return true;
  }

  Future<List<ModeratorEntry>?> replyEntry(
      String shortLink, String replyText, String ppShortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return null;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    return await _sites[surl]!
        .newReply(replyText, target.shortLink, ppShortLink);
  }

  Future<List<ModeratorEntry>?> refreshThread(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    return await _sites[surl]!.refreshThreadViaHTTP(shortLink);
  }

  Future<Uint8List> pullIPFScid(String cid, int xx) async {
    if (cid.isEmpty) return Uint8List(0);
    return _sites[homeSite]!.xxC.pullIPFScid(cid, xx);
  }

  ModeratorEntry? findModeratorEntryByShortLink(String shortLink) {
    for (var i in _sites.values) {
      final t = i.findModeratorEntryByShortLink(shortLink);
      if (t != null) return t;
    }
    return null;
  }

  Future<bool> get refreshAllViaHTTP async {
    allSites.forEach((element) async {
      await element.refreshViaHTTP();
    });
    return true;
  }
}
